/* FD - A program to dump files in hex.                              */

/* SUGS is a set of utilities for DOS that I developed during the    */
/* last few years.  Some of the programs were developed to provide   */
/* utilities missing in native DOS and others were developed just    */
/* for fun.  I used a couple of these programs all the time.  Maybe  */
/* you will find one or two of the programs useful in your work...   */
/*                                                                   */
/*       Hale Landis                                                 */
/*       PO Box 205                                                  */
/*       Louisville, CO 80027                                        */
/*                                                                   */
/* NOTE !  These programs are provided "as is" and you can't hold    */
/* me, Hale Landis, responsible for anything that happens to you or  */
/* your data or your system if you use them.  You may make all the   */
/* copies of the programs you need.  You may pass them on to your    */
/* friends (and enemies).  You may NOT charge anyone for a copy of   */
/* these programs (with the following exception: if someone insists  */
/* on paying you, you must send me 359.9472618 times the amount you  */
/* receive).  Don't use these programs in that product you are about */
/* to starting selling (the product that is going to make you as     */
/* rich and famous as IBM) without cutting me in on the action !     */
/* AND DON'T YOU DARE PUT THEM ON A BULLETIN BOARD !                 */

/* Compile with Lattice C 3.10 using small memory model.             */

#include "stdio.h"
#include "dos.h"

#define MAXLINE 128   /* max line size                       */
#define TABA      0   /* column address starts in            */
#define TABH      7   /* column hex data starts in           */
#define TABTN    44   /* column text starts in - narrow mode */
#define TABEOLN  62   /* column line ends at - narrow mode   */
#define TABTW    81   /* column text starts in - wide mode   */
#define TABEOLW 115   /* column line ends at - wide mode     */

unsigned char s[MAXLINE];           /* output line                    */
unsigned char t[MAXLINE];           /* work area for conversions      */
unsigned char u[MAXLINE];           /* work area for same as above    */
unsigned char *fname;               /* pointer to file name string    */
unsigned char *opt;                 /* pointer to option string       */
FILE *fpi,*fopen();        /* input file pointers            */
int bfile;                 /* binary file if != 0            */
int bpl = 16;              /* bytes per line                 */
int charcntr;              /* number bytes on current line   */
int databyte;              /* the current data byte          */
int dspace;                /* double space if != 0           */
int eof;                   /* found ^z in ascii file if != 0 */
int inrange;               /* within range if != 0           */
int offhex;                /* current offset into hex area   */
int offtext;               /* current offset into text area  */
int sect;                  /* sector number                  */
int stop;                  /* stop dump if != 0              */
int wide;                  /* wide mode if != 0              */
long int addr = -1;        /* address of current byte        */
long int laddrb;           /* start address of line          */
long int laddre;           /* end address of line            */
long int rb = 0;           /* range begin address            */
long int re = 0xffffffl;   /* range end address              */
long int ssab = -1;        /* start of same as above         */
long int ssae;             /* end of same as above           */

#include "sugs.h"       /* sugs special functions */

void gbyte()            /*-----------------------gbyte----------*/

{

/* get the next data byte - if dumping an ascii file, look for */
/* the first ^Z (1A hex) and then set stop on next call.  when */
/* dumping either type of file, set stop if EOF is read or     */
/* increment the data address if a data byte is read.          */

if ( eof )
   stop = 1;
else
   {
   databyte = getc(fpi);
   if ( ! bfile && databyte == '\x1a' ) eof = 1;
   if ( databyte == EOF )
      stop = 1;
   else
      addr++;
   }

}

void bline()            /*-----------------------bline----------*/

{

/* blank and initialize the static data in the output line */

int ibl;

for ( ibl = 0; ibl < MAXLINE; ibl++ ) s[ibl] = ' ';

if ( wide )
   {
   s[TABTW-1] = '*';
   s[TABTW+32] = '*';
   s[TABEOLW] = '\x00';
   }
else
   {
   s[TABTN-1] = '*';
   s[TABTN+16] = '*';
   s[TABEOLN] = '\x00';
   }

}

void pline()            /*-----------------------pline----------*/

{

/* print the output line - if any part of the line is within the */
/* dump range, display it unless it is same as the previous line */

int ipl;

/* check if line is within range */

if (    (laddrb >= rb && laddrb <= re )
     || (laddre >= rb && laddre <= re )
     || (laddrb <= rb && laddre >= re ) )

   /* the line is within the range */

   {

   /* check if line is same as the previous.  if it is, set     */
   /* s.a.a. mode. if it is not and s.a.a mode is set, display  */
   /* the s.a.a. message and reset s.a.a. mode.  display the    */
   /* line if s.a.a. mode is not set.                           */

   for ( ipl = 0; ipl < bpl && u[ipl] == u[ipl+64]; ipl++ ) ;
   if ( ipl == bpl && inrange && ! stop )
      {
      if ( ssab == -1 )
         ssab = laddrb;
      }
   else
      {
      if ( ssab != -1 )
         {
         if ( dspace ) cprintf(" \n");
         cprintf("%06lx TO %06lx SAME AS ABOVE\n",ssab,ssae);
         ssab = -1;
         }
      }
   if ( ssab == -1 )
      {
      if ( dspace ) cprintf(" \n");
      cprintf("%s\n",s);
      }
   inrange = 1;

   }

else

   /* the line is not in the range */

   {

   /* if s.a.a. mode was set, display the s.a.a. message. */

   if ( ssab != -1 )
      {
      if ( dspace ) cprintf(" \n");
      cprintf("%06lx TO %06lx SAME AS ABOVE\n",ssab,ssae);
      ssab = -1;
      }
   inrange = 0;

   }

/* save the line for the next s.a.a. check */

for ( ipl = 0; ipl < bpl; u[ipl+64] = u[ipl++] ) ;
ssae = laddre;

/* blank the output line and reset counters */

bline();
charcntr = 0;
offhex = 0;
offtext = 0;

}

void badopt(ptr)        /*-----------------------badopt---------*/

unsigned char *ptr;

{

/* display the invalid option message */

fprintf(stderr,"\n\x07Invalid option %s ignored !\n",ptr);

}

void range(rp)          /*-----------------------range----------*/

unsigned char *rp;

{

/* process the range addresses in the -r option */

int irange;       /* index into t[]         */
int jrange = 0;   /* != 0 if range invalid  */
int krange = 0;   /* number of ':' in range */

t[0] = '0';
t[64] = '0';

irange = 1;
while ( *rp != '\x00' )
   {
   if ( *rp != ':' )
      t[irange++] = *rp;
   else
      {
      t[irange] = '\x00';
      irange = 65;
      krange++;
      }
   rp++;
   }

t[irange] = '\x00';

irange = 0;
while ( t[irange] != '\x00' )
   {
   if ( irange > 6 || ! isxdigit(t[irange]) ) jrange = 1;
   irange++;
   }

irange = 64;
while ( t[irange] != '\x00' )
   {
   if ( irange > 70 || ! isxdigit(t[irange]) ) jrange = 1;
   irange++;
   }

if ( krange > 1 ) jrange = 1;

if ( ! jrange ) sscanf(&t[0],"%lx",&rb);
if ( ! jrange && krange == 1 ) sscanf(&t[64],"%lx",&re);

if ( jrange || re < rb )
   {
   badopt(opt);
   rb = 0;
   re = 0xffffffl;
   }

}

void main(argc,argv)    /*-----------------------main-----------*/

int argc;
unsigned char *argv[];

{

int imain;
int jmain = 0;

/* get the current DOS switch character */

switchar();

/* get the command line parameters */

for ( imain=1; imain<argc; imain++ )
   {
   opt = argv[imain];
   if ( *opt != schar )
      {
      if ( jmain )
         badopt(opt);
      else
         jmain = imain;
      }
   else
      {
      if ( strlen(opt) >= 2 )
         databyte = toupper(*(opt+1));
      else
         databyte = ' ';
      if (    ( databyte == 'R' && strlen(opt) <= 2 )
           || ( databyte != 'R' && strlen(opt) != 2 ) ) databyte = ' ';
      switch(databyte)
         {
         case 'A': bfile = 0; break;
         case 'B': bfile = 1; break;
         case 'D': dspace = 1; break;
         case 'N': wide = 0; bpl = 16; break;
         case 'R': range(opt+2); break;
         case 'S': dspace = 0; break;
         case 'W': wide = 1; bpl = 32; break;
         default: badopt(opt); break;
         }
      }
   }

/* open the file to be dumped.                            */
/* quit if the file can't be opened.                      */

if ( jmain )
   {
   fname = argv[jmain];
   if ( (fpi = fopen(fname,"rb") ) == NULL )
      {
      fprintf(stderr,"\n\x07Unable to open file %s !\n",fname);
      exit(0);
      }
   }
else
   {
   fprintf(stderr,"\n\x07File not specified !\n");
   exit(0);
   }

/* display the starting messages, blank the output line, */
/* and get the first data byte of the file.              */

if ( bfile )
   cprintf("\nDump of Binary File %s.\n",fname);
else
   cprintf("\nDump of Ascii File %s.\n",fname);

if ( rb != 0 || re != 0xffffffl )
   cprintf("\nRange %lx to %lx will be dumped.\n",rb,re);

cprintf(" \n");

bline();

gbyte();

/* start of main loop */

while ( ! stop )

   {

   /* if the output line is full, display it */

   if ( charcntr == bpl ) pline();

   /* print the sector number every 512 bytes */

   if ( addr % 512 == 0 )
      {
      if ( addr >= rb && addr <= re )
         {
         if ( dspace ) cprintf(" \n");
         cprintf("Sector %d:\n",sect);
         }
      sect++;
      }

   /* if first byte of output line, put address into line */

   if ( addr % bpl == 0 )
      {
      sprintf(t,"%06lx",addr);
      s[TABA+0] = t[0];
      s[TABA+1] = t[1];
      s[TABA+2] = t[2];
      s[TABA+3] = t[3];
      s[TABA+4] = t[4];
      s[TABA+5] = t[5];
      laddrb = addr;
      laddre = addr + bpl - 1;
      }

   /* save the data byte for the same as above test */

   u[charcntr] = databyte;

   /* convert data byte to hex and put into output line */

   sprintf(t,"%02x",databyte);
   s[TABH+offhex++] = t[0];
   s[TABH+offhex++] = t[1];
   if ( addr % 4 == 3 ) s[TABH+offhex++] = ' ';

   /* convert the data byte to character and put into output line */

   if ( databyte < ' ' || databyte > '~' ) databyte = '.';
   if ( wide )
      s[TABTW+offtext++] = databyte;
   else
      s[TABTN+offtext++] = databyte;

   /* increment the character count and get the next character */

   charcntr++;
   gbyte();

   /* end of main loop */

   }

/* display the last output line */

pline();

/* display the ending messages */

if ( bfile )
   cprintf("\nEnd-of-file.\n");
else
   {
   if ( eof )
      cprintf("\nEnd-of-File (^Z, 1A hex) found.\n");
   else
      cprintf("\nEnd-of-File (^Z, 1A hex) not found.\n");
   }
cprintf("File %s contains %lx hex (%ld decimal) bytes(s).\n",
        fname,addr+1,addr+1);
cprintf("File %s is %d sector(s) long.\n",
        fname,sect);

/* done - exit */

exit(0);

}

/* end of FD.C source file */
